#include <SoftwareSerial.h>
#include <Wire.h>
#include <SHT2x.h>
#include "Adafruit_SI1145.h"
#include <SparrowTransfer.h>

#define ESP8266_DEBUG
#include "coap.h"

ESP8266 wifi = ESP8266(Serial1);
Coap coap(wifi);

#define SSID "TODO"
#define PASS "TODO"

IPAddress ip;
int controlPin = 8;
int ledPin = 11;
bool LEDSTATE;

void callback_light(CoapPacket &packet, IPAddress ip, int port) {
  Serial.println("[Light] ON/OFF");

  char p[packet.payloadlen + 1];
  memcpy(p, packet.payload, packet.payloadlen);
  p[packet.payloadlen] = NULL;
  
  String message(p);
  Serial.println(message);

  if (message.equals("0"))
    LEDSTATE = false;
  else if(message.equals("1"))
    LEDSTATE = true;
      
  if (LEDSTATE) {
    digitalWrite(11, HIGH) ; 
    coap.sendResponse(ip, port, packet.messageid, "1");
  } else { 
    digitalWrite(11, LOW) ; 
    coap.sendResponse(ip, port, packet.messageid, "0");
  }
  
}

void callback_response(CoapPacket &packet, IPAddress ip, int port) {
  Serial.println("[Coap Response got]");
  
  char p[packet.payloadlen + 1];
  memcpy(p, packet.payload, packet.payloadlen);
  p[packet.payloadlen] = NULL;
  
  Serial.println(p);
}

void setup()
{
  pinMode(11, OUTPUT);
  digitalWrite(11, HIGH);
  pinMode(controlPin, OUTPUT);  //sensor on/off control
  digitalWrite(controlPin, HIGH);

  pinMode(12, OUTPUT);
  digitalWrite(12, HIGH);
  
  Serial.begin(57600);
  Serial1.begin(9600);
  wifi.begin();
  wifi.setTimeout(1000);


  Serial.println("starting");
  Serial.println(wifi.restart());
  Serial.println(getStatus(wifi.setMode(ESP8266_WIFI_BOTH)));
  Serial.println(getStatus(wifi.joinAP(SSID, PASS)));
  Serial.println(getStatus(wifi.getIP(ESP8266_WIFI_BOTH, ip)));
  Serial.println(ip);
  Serial.println(getStatus(wifi.setMultipleConnections(true)));
  Serial.println(getStatus(wifi.createServer(8083)));

  Serial.println("Setup Callback Light");
  coap.server(callback_light, "light123");

  Serial.println("Setup Response Callback");
  coap.response(callback_response);

  coap.start();
}

void loop()
{
  delay(1000);
  coap.loop();

}

String getStatus(ESP8266CommandStatus status)
{
    Serial.print("my:");
    switch (status) {
    case ESP8266_COMMAND_INVALID:
        return "INVALID";
        break;

    case ESP8266_COMMAND_TIMEOUT:
        return "TIMEOUT";
        break;

    case ESP8266_COMMAND_OK:
        return "OK";
        break;

    case ESP8266_COMMAND_NO_CHANGE:
        return "NO CHANGE";
        break;

    case ESP8266_COMMAND_ERROR:
        return "ERROR";
        break;

    case ESP8266_COMMAND_NO_LINK:
        return "NO LINK";
        break;

    case ESP8266_COMMAND_TOO_LONG:
        return "TOO LONG";
        break;

    case ESP8266_COMMAND_FAIL:
        return "FAIL";
        break;

    default:
        return "UNKNOWN COMMAND STATUS";
        break;
    }
}
